import os, time
from os.path import *
import gnomeapplet, gtk, gtk.gdk, gconf, gobject
from gettext import gettext as _
import gtk, gtk.glade, egg.trayicon, gobject, gnomevfs
import csv, os
from gettext import gettext as _
import invest, invest.about, invest.chart, invest.quotes
from invest import *

class InvestWidget(gtk.TreeView):
	def __init__(self):
		gtk.TreeView.__init__(self)
				
		self.set_property("headers-visible", False)
		self.set_property("rules-hint", True)
		self.set_reorderable(True)
		
		cell_description = gtk.CellRendererText ()
		self.column_description = gtk.TreeViewColumn ("Description", cell_description)
		self.column_description.set_attributes (cell_description, markup=0)

		self.append_column(self.column_description)
		self.connect('row-activated', self.on_row_activated)		
		self.refresh()
		
		gobject.timeout_add(AUTOREFRESH_TIMEOUT, lambda: invest.quotes.get_quotes(self.on_quotes_received))
	
	def refresh(self):
		invest.quotes.get_quotes(self.on_quotes_received, True)
		
	def set_toggle_button(self, tb):
		self.tb = tb
				
	def on_quotes_received(self, quotes):
		current = sum([invest.STOCKS[ticker]["amount"]*val["trade"] for ticker, val in quotes.items()])
		paid = sum([invest.STOCKS[ticker]["amount"]*invest.STOCKS[ticker]["bought"] for ticker in quotes.keys()]) + 2*invest.STOCKS[ticker]["comission"]*len(invest.STOCKS)
		balance = current - paid

		color = "green"
		if balance < 0:
			color = "red"
		
		store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
		self.set_model(store)
				
		store.append(["<span face='Monospace'>%s: <span foreground='%s'>%+.2f$ (%+.2f%%)</span> %.2f$</span>" % (_("Total"), color, balance, balance/paid*100, current), None])
		for ticker, val in quotes.items():
			current = invest.STOCKS[ticker]["amount"]*val["trade"]
			paid = invest.STOCKS[ticker]["amount"]*invest.STOCKS[ticker]["bought"] + 2*invest.STOCKS[ticker]["comission"]
			balance = current - paid
			
			color = "green"
			if balance < 0:
				color = "red"
							
			store.append(["<span face='Monospace'>%s:  <span foreground='%s'>%+.2f$ (%+.2f%%)</span> %.2f$</span>" % (ticker, color, balance, balance/paid*100, val["trade"]), ticker])
				
	def on_row_activated(self, treeview, path, view_column):
		ticker = self.get_model()[self.get_model().get_iter(path)][1]
		if ticker == None:
			return
		
		invest.chart.show_chart([ticker])
		self.tb.close()
		
class InvestTicker(gtk.Label):
	def __init__(self):
		gtk.Label.__init__(self, _("Waiting..."))
		
		self.quotes = []
		self.refresh()
		
		gobject.timeout_add(AUTOREFRESH_TIMEOUT, lambda: invest.quotes.get_quotes(self.on_quotes_received))
		gobject.timeout_add(TICKER_TIMEOUT, self.scroll_quotes)
	
	def refresh(self):
		invest.quotes.get_quotes(self.on_quotes_received, True)
		
	def scroll_quotes(self):
		if len(self.quotes) == 0:
			return True
		
		q = self.quotes.pop()
		self.set_markup("<span face='Monospace'>%s</span>" % q)
		self.quotes.insert(0, q)
		
		return True
		
	def on_quotes_received(self, quotes):
		current = sum([invest.STOCKS[ticker]["amount"]*val["trade"] for ticker, val in quotes.items()])
		paid = sum([invest.STOCKS[ticker]["amount"]*invest.STOCKS[ticker]["bought"] for ticker in quotes.keys()]) + 2*invest.STOCKS[ticker]["comission"]*len(invest.STOCKS)
		balance = current - paid

		color = "green"
		if balance < 0:
			color = "red"
		
		self.quotes.append("%s: <span foreground='%s'>%+.2f$ (%+.2f%%)</span> %.2f$" % (_("Total"), color, balance, balance/paid*100, current))
		for ticker, val in quotes.items():
			current = invest.STOCKS[ticker]["amount"]*val["trade"]
			paid = invest.STOCKS[ticker]["amount"]*invest.STOCKS[ticker]["bought"] + 2*invest.STOCKS[ticker]["comission"]
			balance = current - paid
			
			color = "green"
			if balance < 0:
				color = "red"
							
			self.quotes.append("%s:  <span foreground='%s'>%+.2f$ (%+.2f%%)</span> %.2f$" % (ticker, color, balance, balance/paid*100, val["trade"]))
		
		s="%s:\n%+.2f$ (%+.2f%%)\n" % (_("Total"), balance, balance/paid*100)
		for ticker, val in quotes.items():
			current = invest.STOCKS[ticker]["amount"]*val["trade"]
			paid = invest.STOCKS[ticker]["amount"]*invest.STOCKS[ticker]["bought"] + 2*invest.STOCKS[ticker]["comission"]
			balance = current - paid
			s += ("\n%s:\n%+.2f$ (%+.2f%%)" % (ticker, balance, balance/paid*100))
		
		invest.dbusnotification.notify(
			_("Financial Report"),
			"stock_chart",
			_("Financial Report"),
			s,
			10000)
